// EEPROMのファイルリストを表示する
// 上下ボタンでファイルスロットを選択すると
// 内容を16x16のドットイメージとして表示します
// STARTボタンで実行します

' 色定数の定義
WHITE = rgb(255, 255, 255)
GREEN = rgb(0, 255, 0)
ORANGE = rgb(255, 163, 0)
DARKBLUE = rgb(0, 0, 64)

' ボタンの定義
BUTTON_UP = 5
BUTTON_DOWN = 8
BUTTON_A = 11
BUTTON_B = 10
BUTTON_START = 9

' 変数初期化
Ymax=19
Smax=31-Ymax
y=0
s=0
Cmin=3
Cmax=Ymax-3

' スプライトパターン定義
CS01 = getmptr()
mdata 0xc0,0xf0,0xfc,0xff,0xfc,0xf0,0xc0,0

' ワークエリア定義
OFFSET = getmptr()
mptr OFFSET + 2
BUF = getmptr()
mptr BUF + 256

i2ci 1,400

' 初期画面 描画
cls 2
gprint 0,0,"ORANGE-inside  File Launcher",GREEN,0

window(0,1,20,20)
view_list(s)

window(22,1,16,16)
view_image(s+y)

window(22,23,16,5)
gprint 23*8,24*8,"UP   : FileNo.-1",ORANGE,0
gprint 23*8,25*8,"DOWN : FileNo.+1",ORANGE,0
gprint 23*8,26*8,"START: RUN File",ORANGE,0
gprint 23*8,27*8,"A    : Download",ORANGE,0
gprint 23*8,28*8,"B    : DUMP HEX",ORANGE,0

' スプライト作成・表示
sprnew(1,8,(y+2)*8,8,8,CS01,WHITE,0)
sprdsp 1,1

' スタートボタン リリース待ち
while !in(BUTTON_START)
wend

' メインループ
while in(BUTTON_START)
	d = up_check() | down_check()
	if trig_a() then eeprom_download(s+y):d=1
	if trig_b() then eeprom_dump(s+y)
	if d&1 then view_list(s)
	if d&2 then sprmov 1,8,(y+2)*8
	if d then view_image(s+y)
wend

' スタートボタン リリース待ち
while !in(BUTTON_START)
wend

chain s+y+1

end

' カーソル上移動チェック スクロール付
def up_check() {
	if !trig_up() then return 0
	if (y=Cmin) and (s>0) then s=s-1:return 1
	y=y-1:if y<0 then s=Smax:y=Ymax:return 3
	return 2
}

' カーソル下移動チェック スクロール付
def down_check() {
	if !trig_down() then return 0
	if (y=Cmax) and (s<Smax) then s=s+1:return 1
	y=y+1:if y>Ymax then s=0:y=0:return 3
	return 2
}

' ファイルリストを表示
def view_list(n) {
	var i
	rect 2*8,2*8,19*8,20*8,0,1
	for i=0 to Ymax
		eeprom_read((i+n)*2048,BUF,32)
		gp_num(2, i+2, GREEN, n+i+1, 2)
		if mpeekw(BUF/2) != 0x7069 then continue
		gp_mem(5, i+2, BUF+4, 8)
		gp_num(13, i+2, ORANGE, mpeekw((BUF+22)/2), 4)
		gp_num(17, i+2, WHITE,  mpeekw((BUF+30)/2), 4)
	next
}

' EEPROMから読み込んで16x16イメージとして表示
def view_image(n) {
	var x
	for x=0 to 7
		eeprom_read(n*2048 + x*256 ,BUF ,256)
		sprnew(0,184+x*16,16,16,128,BUF,WHITE,DARKBLUE)
		sprdsp 0,1
	next
}

' ファイルをHEXデータとしてシリアルコンソールに出力する
def eeprom_dump(n) {
	var i
	var len
	var ptr
	var r

	eeprom_read(n*2048,BUF,32)
	len = 32+ mpeekw(BUF/2 + 11) + mpeekw(BUF/2 + 15)
// len = 2048

	print "D"

	ptr=0
	while len
		r=32
		if len<32 then r=len
		eeprom_read(n*2048 + ptr,BUF,r)
		for i=0 to r-1
			print hex$(mpeek(BUF+i));
			if (i&15)=15 then print
		next
		len = len - r
		ptr = ptr + r
	wend
	print "$"

	print "R"
}

' シリアルコンソールからHEXデータを受信しEEPROMに書き込む
def eeprom_download(n) {
	var len
	var ptr
	var r
	var c
	var d
	var e

	len = 0
	ptr = 0
	r = 0
	c = 0
	e = -1

' 受信バッファクリア
	while inkey()>=0
	wend
	print "Ready to receive"

	window(7,12,12,1)
	gprint 8*8,13*8,"Receiving...",WHITE,0

	while (c != 0x24) && !trig_a()
		c=inkey()
' エコーバック
		if c>0 then print chr$(c);
		d = -1
		if (c >= 48) and (c <= 57) then d= c-48
		if (c >= 65) and (c <= 70) then d= c-55
		if d<0 then continue
' 1文字受信
		if e<0 then e = d*16 : continue
' 1バイト受信
		if len >= 2048 then print "Too Large!" : break
		mpoke BUF + r, e + d
		e = -1
		r = r + 1
		if r<32 then continue
' 32バイト受信
		eeprom_write(n*2048 + ptr)
		len = len + r
		ptr = ptr + r
		r = 0
	wend
' 32バイトに満たない受信分を書きだし
	if r then eeprom_write(n*2048 + ptr) : len = len + r

	print
	print len;" Bytes"
	return len
}

' EEPROMからptrにlenバイト分 読み込む
def eeprom_read(adr,ptr,len) {
	var e
	mpokew OFFSET/2,adr
	e = i2cw(1,0x50,OFFSET,2,0)
	e = i2cr(1,0x50,ptr,len,1)
}

' EEPROMにBUFを32バイト分 書き込む
def eeprom_write(adr) {
	var e
	mpokew OFFSET/2,adr
	e = i2cw(1,0x50,OFFSET,34,1)
}

' メモリー配列内の文字列を n文字分 表示
def gp_mem(x,y,a,n) {
	while n
		gp(x, y, mpeek(a))
		a = a + 1
		x = x + 1
		n = n - 1
	wend
}

' 数値を n桁分 表示
def gp_num(x,y,col,d,n) {
	var first
	first = 1
	while n
		if d || first then gprint (x+n-1)*8, y*8, d%10, col, 0
		d = d / 10
		first = 0
		n = n - 1
	wend
}
' 枠線を描く
def window(x,y,w,h) {
	var i
	var ex
	var ey
	ex = x+w+1
	ey = y+h+1

	gp(x, y,0x9C)
	gp(x,ey,0x9E)
	for i=1 to w
		gp(x+i, y,0x97)
		gp(x+i,ey,0x97)
	next
	gp(ex, y,0x9D)
	gp(ex,ey,0x9F)

	for i=1 to h
		gp( x,y+i,0x96)
		gp(ex,y+i,0x96)
	next

	rect (x+1)*8,(y+1)*8,w*8,h*8,0,1
}

def gp(x,y,c) {
	gprint x*8,y*8,chr$(c),WHITE,0

}
' ボタン状態が変化＆押された時だけ1になる関数
def trig_down() {
	var ret
	ret = pre_d
	pre_d = in(BUTTON_DOWN)
	return ret & !pre_d
}
def trig_up() {
	var ret
	ret = pre_u
	pre_u = in(BUTTON_UP)
	return ret & !pre_u
}
def trig_a() {
	var ret
	ret = pre_a
	pre_a = in(BUTTON_A)
	return ret & !pre_a
}
def trig_b() {
	var ret
	ret = pre_b
	pre_b = in(BUTTON_B)
	return ret & !pre_b
}